@@ -1,4 +1,4 @@
-/* $Id: debug.cpp 26506 2014-04-24 19:51:45Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,6 +17,10 @@
 #include "fileio_func.h"
 #include "settings_type.h"
 
+#if defined(_WIN32)
+#include "os/windows/win32.h"
+#endif
+
 #include <time.h>
 
 #if defined(ENABLE_NETWORK)
@@ -100,8 +104,6 @@ char *DumpDebugFacilityNames(char *buf, char *last)
 	return buf;
 }
 
-#if !defined(NO_DEBUG_MESSAGES)
-
 /**
  * Internal function for outputting the debug line.
  * @param dbg Debug category.
@@ -137,10 +139,10 @@ static void debug_print(const char *dbg, const char *buf)
 	} else {
 		char buffer[512];
 		seprintf(buffer, lastof(buffer), "%sdbg: [%s] %s\n", GetLogPrefix(), dbg, buf);
-#if defined(WINCE)
-		NKDbgPrintfW(OTTD2FS(buffer));
-#elif defined(WIN32) || defined(WIN64)
-		_fputts(OTTD2FS(buffer, true), stderr);
+#if defined(_WIN32)
+		TCHAR system_buf[512];
+		convert_to_fs(buffer, system_buf, lengthof(system_buf), true);
+		_fputts(system_buf, stderr);
 #else
 		fputs(buffer, stderr);
 #endif
@@ -168,7 +170,6 @@ void CDECL debug(const char *dbg, const char *format, ...)
 
 	debug_print(dbg, buf);
 }
-#endif /* NO_DEBUG_MESSAGES */
 
 /**
  * Set debugging levels by parsing the text in \a s.
